// File        : daqpath.v     Author    : Tullio Grassi           
// NB: sfifo36x512 has no stable data the first cycle after empty -> 0 !!
// thus any module using it, must delay empty.
// 
// 11 Feb 2004: I start implementing the Zero-Sup. 
// Need to add Threshold1-24 in the DaqPath.v interface. 
// Then I should drop DaqPath.v and its submodules into the next version,
// or comparing this DaqPath.v and its submodules with the current version.
// The control of data flow must change: DaqFormatter cannot start as
// soon as EventBuffer is not empty, as DaqFormatter now sucks data 
// faster they they are written (comparisons slow downs writing).

`include "define.v"
module DaqPath  (
 input  clk,  RST,
 input Fib1BZero, Fib2BZero, Fib3BZero, Fib4BZero, 
       Fib5BZero, Fib6BZero, Fib7BZero, Fib8BZero, // one BZero per fiber
       L1A, CalibTrig, CounterModeSel, LW, StRq,  TTC_Ready,
       PatternMode, CompactNorm, CompactCalib,  DInError, DLL_lock,
 input  [10:0]
       QIE1Data,  QIE2Data,  QIE3Data,  QIE4Data,  QIE5Data,  QIE6Data,   
       QIE7Data,  QIE8Data,  QIE9Data,  QIE10Data, QIE11Data, QIE12Data,
       QIE13Data, QIE14Data, QIE15Data, QIE16Data, QIE17Data, QIE18Data,  
       QIE19Data, QIE20Data, QIE21Data, QIE22Data, QIE23Data, QIE24Data,
       TP1_Data,  TP2_Data,  TP3_Data,  TP4_Data,  TP5_Data,  TP6_Data,  
       TP7_Data,  TP8_Data,  TP9_Data,  TP10_Data, TP11_Data, TP12_Data,
       TP13_Data, TP14_Data, TP15_Data, TP16_Data, TP17_Data, TP18_Data, 
       TP19_Data, TP20_Data, TP21_Data, TP22_Data, TP23_Data, TP24_Data,
 input  [17:0] 
          Threshold1,  Threshold2,  Threshold3,  Threshold4, 
          Threshold5,  Threshold6,  Threshold7,  Threshold8, 
          Threshold9,  Threshold10, Threshold11, Threshold12, 
          Threshold13, Threshold14, Threshold15, Threshold16, 
          Threshold17, Threshold18, Threshold19, Threshold20, 
          Threshold21, Threshold22, Threshold23, Threshold24, 
 input [8:1] FifoEmpty, FifoFull, // Conditions of the 8 Input FIFOs.
 input [7:0] PipeLength, // L1A-latency Pipeline length; 
                        // spec from Trigger TDR page 563 : < 140 bx
 input [4:0] NS, NPS, // Number of Samples and Pre-Samples per L1A
 input [3:0] Fifo1Status, Fifo2Status, Fifo3Status, Fifo4Status, 
             Fifo5Status, Fifo6Status, Fifo7Status, Fifo8Status,   
 input [1:0] UnlockCnt,
 input [`EVN_WIDTH - 1 : 0]  EVN, CTrEvN,
 input [`BCN_WIDTH - 1 : 0]  BCN,    
 input [`OrN_WIDTH - 1 : 0]  OrN,
 input [`HCN_WIDTH - 1 : 0]  HCN,
 output legal_L1A, BZ, TrigMode_from_pf,
        EmEv_from_pf, PointerFifoEmpty, EventBufferPop, 
        EventBufferEmpty, EventBufferPush,
 output [`DAQ_MUX_SEL-1:0]   ChanSel, 
 output [`DRND_ADDR_WIDTH-1:0]  DrndRdAddr, 
 output reg  [`MAIN_DATA_WIDTH - 1 : 0]   DataOut,     
 output reg  [1:0]    StatusBits,
 output [23:0]  L2frameINT,  // DAQ-path output before series resistors.
 output reg  [`BCN_WIDTH-1:0]
    BCNofFib1BZero, BCNofFib2BZero, BCNofFib3BZero, BCNofFib4BZero, 
    BCNofFib5BZero, BCNofFib6BZero, BCNofFib7BZero, BCNofFib8BZero,
 output  reg   L2IDLE_b    );
     
//    ***********   INTERNAL SIGNALS   ************
  reg  [`EVN_WIDTH - 1 : 0]  EVN_D, CTrEvN_D; // delayed versions of 
  reg  [`BCN_WIDTH - 1 : 0]  BCN_D; // EVN, BCN, OrN  in order to match  
  reg  [`OrN_WIDTH - 1 : 0]  OrN_D; // the delay L1A -> legal_L1A  
  reg  CalibTrig_D2, CalibTrig_D; 
  wire DrndWe,  FifosPop, EventBufferFull,
       OW,  EmEv_to_fifo, RL_to_fifo,  RL_from_pf; 
  wire [`DRND_ADDR_WIDTH-1:0]  DrndWrAddr, PointerOut; 
  wire [`EVN_WIDTH - 1 : 0]   EvnOut, CTrEvNOut;
  wire [`BCN_WIDTH - 1 : 0]   BcnOut;
  wire [`OrN_WIDTH - 1 : 0]   OrnOut;
  wire [1:0]  StatusBitsInt, StatusBitsFake, StatusBitsReal;
  wire [7:0] TPwordCount;
  wire [8:0] DAQwordCount;
  wire [`MAIN_DATA_WIDTH - 1 : 0] DataOutInt, PayloadData, DataOutReal,
                                  DaqMuxOut, DataOutFake, CounterData;
  wire [19:0] pad20;   // just to fill the event_buffer outputs
  wire [15:0] pad16;   // just to fill the outputs of *N_FIFO    
  wire [11:0] pad12a;  // just to fill the outputs of *N_FIFO    
 
//  *************************************************************
//  here we mark the arrival time of the last BZero from each FE-fiber:
 always@(posedge clk) begin
  if(RST) begin 
    BCNofFib1BZero <= 0; BCNofFib2BZero <= 0; BCNofFib3BZero <= 0; BCNofFib4BZero <= 0; 
    BCNofFib5BZero <= 0; BCNofFib6BZero <= 0; BCNofFib7BZero <= 0; BCNofFib8BZero <= 0;
  end
 else begin
   if (Fib1BZero)    BCNofFib1BZero <= BCN;
   if (Fib2BZero)    BCNofFib2BZero <= BCN;         
   if (Fib3BZero)    BCNofFib3BZero <= BCN;
   if (Fib4BZero)    BCNofFib4BZero <= BCN;
   if (Fib5BZero)    BCNofFib5BZero <= BCN;
   if (Fib6BZero)    BCNofFib6BZero <= BCN;
   if (Fib7BZero)    BCNofFib7BZero <= BCN;
   if (Fib8BZero)    BCNofFib8BZero <= BCN;
 end
 end // always

 PipeDrnd     PipeDrnd  // includes L1A-latency Pipeline and Derand. Buffer  
 ( .clk(clk),                 .DInError(DInError),       .DInErDel(DInErDel),
   .data_in_1_ch(QIE1Data),   .data_in_2_ch(QIE2Data),   .data_in_3_ch(QIE3Data), 
   .data_in_4_ch(QIE4Data),   .data_in_5_ch(QIE5Data),   .data_in_6_ch(QIE6Data), 
   .data_in_7_ch(QIE7Data),   .data_in_8_ch(QIE8Data),   .data_in_9_ch(QIE9Data), 
   .data_in_10_ch(QIE10Data), .data_in_11_ch(QIE11Data), .data_in_12_ch(QIE12Data),
   .data_in_13_ch(QIE13Data), .data_in_14_ch(QIE14Data), .data_in_15_ch(QIE15Data), 
   .data_in_16_ch(QIE16Data), .data_in_17_ch(QIE17Data), .data_in_18_ch(QIE18Data), 
   .data_in_19_ch(QIE19Data), .data_in_20_ch(QIE20Data), .data_in_21_ch(QIE21Data), 
   .data_in_22_ch(QIE22Data), .data_in_23_ch(QIE23Data), .data_in_24_ch(QIE24Data), 
   .TP1_Data(TP1_Data),    .TP2_Data(TP2_Data),    .TP3_Data(TP3_Data), 
   .TP4_Data(TP4_Data),    .TP5_Data(TP5_Data),    .TP6_Data(TP6_Data), 
   .TP7_Data(TP7_Data),    .TP8_Data(TP8_Data),    .TP9_Data(TP9_Data), 
   .TP10_Data(TP10_Data),  .TP11_Data(TP11_Data),  .TP12_Data(TP12_Data), 
   .TP13_Data(TP13_Data),  .TP14_Data(TP14_Data),  .TP15_Data(TP15_Data), 
   .TP16_Data(TP16_Data),  .TP17_Data(TP17_Data),  .TP18_Data(TP18_Data),
   .TP19_Data(TP19_Data),  .TP20_Data(TP20_Data),  .TP21_Data(TP21_Data), 
   .TP22_Data(TP22_Data),  .TP23_Data(TP23_Data),  .TP24_Data(TP24_Data), 
   .DrndWrAddr(DrndWrAddr), .DrndRdAddr(DrndRdAddr), .DrndWe(DrndWe), 
   .ChanSel(ChanSel), .DaqMuxOut(DaqMuxOut), .PipeLength(PipeLength), .NPS(NPS) );
    
 DaqCntr  DaqCntr
   (.clk(clk), .rst(RST), .BZ(BZ),  .OW(OW),
    .L1A(L1A),  .CalibTrig(CalibTrig),  .NS(NS), .NPS(NPS),  
    .pendingL1A_b(PointerFifoEmpty),  .DrndRdAddr(DrndRdAddr), 
    .DrndWe(DrndWe),   .DrndWrAddr(DrndWrAddr),
    .EvtRdy4Tx(EvtRdy4Tx),  .RdyAck(RdyAck),
    .EventBufferEmpty(EventBufferEmpty),  .ChanSel(ChanSel), 
    .EventBufferPush(EventBufferPush), .pendingL1A_20d(),
    .EmEv_to_fifo(EmEv_to_fifo), .RL_to_fifo(RL_to_fifo),
    .DaqMuxOut(DaqMuxOut[7:0]),  .DrndPointer(PointerOut), 
    .Threshold1(Threshold1),     .Threshold2(Threshold2), 
    .Threshold3(Threshold3),     .Threshold4(Threshold4), 
    .Threshold5(Threshold5),     .Threshold6(Threshold6), 
    .Threshold7(Threshold7),     .Threshold8(Threshold8), 
    .Threshold9(Threshold9),     .Threshold10(Threshold10), 
    .Threshold11(Threshold11),   .Threshold12(Threshold12), 
    .Threshold13(Threshold13),   .Threshold14(Threshold14), 
    .Threshold15(Threshold15),   .Threshold16(Threshold16), 
    .Threshold17(Threshold17),   .Threshold18(Threshold18), 
    .Threshold19(Threshold19),   .Threshold20(Threshold20), 
    .Threshold21(Threshold21),   .Threshold22(Threshold22), 
    .Threshold23(Threshold23),   .Threshold24(Threshold24), 
    .legal_L1A(legal_L1A),       .EmEv_from_pf(EmEv_from_pf),
    .TPwordCount(TPwordCount),   .DAQwordCount(DAQwordCount)    );

// The following 3 FIFOs store in parallel some qualifiers of each legal_L1A. 
// EVN, BCN, OrN are relative to L1A (input of DaqPath) and
// have to be delayed as much as legal_L1A is (2 ticks).
// Comparisons with DCC say BCN is delayed too much (1 tick), so use *_D
// The pointers are the address of the derandomizer 
// of the first time-sample of each event.

 always@(posedge clk) begin  // dealy the various trigger qualifiers 
      EVN_D <=  EVN; CTrEvN_D <=  CTrEvN;  BCN_D <=  BCN; OrN_D <=  OrN; 
      CalibTrig_D2 <= CalibTrig_D;  CalibTrig_D <= CalibTrig; 
   end 

 sfifo36x512      EVN_FIFO  
  ( .clk(clk),                        .rst(RST),     
    .push(legal_L1A),                 .pop(FifosPop),
    .data_in({12'b0, EVN_D}),         .data_out({pad12a, EvnOut}),
    .full(),                          .empty()   );
 sfifo36x512      BCN_OrN_FIFO  
  ( .clk(clk),                        .rst(RST),     
    .push(legal_L1A),                 .pop(FifosPop),
    .data_in({16'b0, BCN_D, OrN_D}),  .data_out({pad16, BcnOut, OrnOut}),
    .full(),                          .empty()          );  
 sfifo36x512    pointerfifo
  ( .clk(clk),                        .rst(RST),     
    .push(legal_L1A),                 .pop(FifosPop),
    .data_in( {CTrEvN_D, CalibTrig_D2, EmEv_to_fifo, RL_to_fifo, DrndWrAddr}),
    .data_out({CTrEvNOut, TrigMode_from_pf, EmEv_from_pf, RL_from_pf, PointerOut}),
    .full(),      .empty(PointerFifoEmpty)     );
 // the full of pointerfifo is never used, as the first buffer to overflow is drnd_buffer

//this module buffers the data from the derandomizer          
   sfifo36x512    EventBuffer  
   ( .clk(clk),                      .rst(RST),     
     .push(EventBufferPush),         .pop(EventBufferPop),
     .data_in({20'b0, DaqMuxOut}),   .data_out({pad20, PayloadData}),
     .full(EventBufferFull),         .empty(EventBufferEmpty)     );

// this module receive data from the event buffer and creates the frame
DaqFormatter  DaqFormReal 
 (  .clk(clk),   .rst(RST),  .data_in(PayloadData),     
    .CompactNorm(CompactNorm), .CompactCalib(CompactCalib),
    .TrigMode(TrigMode_from_pf), .EvN(EvnOut), .CTrEvN(CTrEvNOut),
    .bcn(BcnOut), .orn(OrnOut),  .NS(NS),   .NPS(NPS),  
    .TPwordCount(TPwordCount),   .DAQwordCount(DAQwordCount), 
    .htr_card_no(HCN),  .PipeLength(PipeLength), 
    .StatusBits (StatusBitsReal), .DataOut(DataOutReal),
    .FifosPop(FifosPop),          .EventBufferPop(EventBufferPop), 
    .EventBufferEmpty(EventBufferEmpty), .DInError(DInErDel),
    .EmEv(EmEv_from_pf),  .RL(RL_from_pf), .DLL_lock(DLL_lock),
    .OW(OW),  .BZ(BZ),     .StRq(StRq), 
    .EvtRdy4Tx(EvtRdy4Tx), .RdyAck(RdyAck),
    .FifoFull(FifoFull), .FifoEmpty(FifoEmpty), .LW(LW),
    .Fifo1Status(Fifo1Status),    .Fifo2Status(Fifo2Status), 
    .Fifo3Status(Fifo3Status),    .Fifo4Status(Fifo4Status), 
    .Fifo5Status(Fifo5Status),    .Fifo6Status(Fifo6Status), 
    .Fifo7Status(Fifo7Status),    .Fifo8Status(Fifo8Status), 
    .UnlockCnt(UnlockCnt),        .TTC_Ready(TTC_Ready), 
    .Extra1Info(BCNofFib1BZero),  .Extra2Info(BCNofFib2BZero),
    .Extra3Info(BCNofFib3BZero),  .Extra4Info(BCNofFib4BZero), 
    .Extra5Info(BCNofFib5BZero),  .Extra6Info(BCNofFib6BZero),
    .Extra7Info(BCNofFib7BZero),  .Extra8Info(BCNofFib8BZero),
    .TestMode(PatternMode)                );

// Next two modules to generate counter data. 
// The logic is not reviewd since several versions.
CounterModCntr CounterModCntr (.clk(clk), .CounterData(CounterData));

DaqFormatter DaqFormFake 
  ( .clk(clk),    .rst(RST),       
    .CompactNorm(CompactNorm), .CompactCalib(CompactCalib),
    .TrigMode(TrigMode_from_pf), .EvN(EvnOut), .CTrEvN(CTrEvNOut),
    .bcn(BcnOut),  .orn(OrnOut),      .NS(NS), .NPS(NPS),  
    .TPwordCount(TPwordCount),   .DAQwordCount(DAQwordCount), 
    .htr_card_no(HCN),  .PipeLength(PipeLength), 
    .data_in(CounterData),    // here we inject fake data
    .StatusBits (StatusBitsFake),   .DataOut(DataOutFake),  // fake output
    .FifosPop(),   .EventBufferPop(),   .DInError(DInErDel),
    .EventBufferEmpty(EventBufferEmpty), 
    .EmEv(EmEv_from_pf),    .RL(RL_from_pf), .DLL_lock(DLL_lock),
    .OW(OW),  .BZ(BZ),     .StRq(StRq), 
    .EvtRdy4Tx(1'b1), .RdyAck(),
    .FifoFull(FifoFull), .FifoEmpty(FifoEmpty), .LW(LW),
    .Fifo1Status(Fifo1Status),    .Fifo2Status(Fifo2Status), 
    .Fifo3Status(Fifo3Status),    .Fifo4Status(Fifo4Status), 
    .Fifo5Status(Fifo5Status),    .Fifo6Status(Fifo6Status), 
    .Fifo7Status(Fifo7Status),    .Fifo8Status(Fifo8Status), 
    .UnlockCnt(UnlockCnt),        .TTC_Ready(TTC_Ready), 
    .Extra1Info(BCNofFib1BZero),  .Extra2Info(BCNofFib2BZero),
    .Extra3Info(BCNofFib3BZero),  .Extra4Info(BCNofFib4BZero), 
    .Extra5Info(BCNofFib5BZero),  .Extra6Info(BCNofFib6BZero),
    .Extra7Info(BCNofFib7BZero),  .Extra8Info(BCNofFib8BZero),
    .TestMode(CounterModeSel)     );

 mux18      RealOrFake
  ( .ck(clk),            .sel(CounterModeSel), 
    .Din_One ( {StatusBitsFake, DataOutFake}),
    .Din_Zero( {StatusBitsReal, DataOutReal}), 
    .Mux_Out ( {StatusBitsInt,  DataOutInt} )       );

always@(posedge clk) begin // FFs keep L2IDLE_b aligned with data
  DataOut    <= DataOutInt;
  StatusBits <= StatusBitsInt;
  L2IDLE_b   <= (StatusBitsInt[1] | StatusBitsInt[0]);
   end 

 hamcode_gen    hamcode_gen 
    ( .clk(clk), .D(DataOut), .S(StatusBits),  .L2frame(L2frameINT) );

endmodule      