/* This module generates the hamming code from the incoming data.
This format is required for the LVDS Link.  */

module hamcode_gen  ( clk,  D,   S,  L2frame );

 input  [15:0]  D;  // User Data
 input  [1:0]   S;  // Status Bits 
 input  clk; 
 output [23:0]   L2frame;     reg [23:0]  L2frame;  // Hamming-encoded output
 
 wire        P;    // Parity Bit
 wire [4:0]  H;  

 // HTR/DCC link data format
 always@(posedge clk)
  begin
   L2frame[1:0]   <= H[1:0];
   L2frame[2]     <= D[0];
   L2frame[3]     <= H[2];
   L2frame[6:4]   <= D[3:1];
   L2frame[7]     <= H[3];  
   L2frame[14:8]  <= D[10:4];
   L2frame[15]    <= H[4];  
   L2frame[20:16] <= D[15:11];
   L2frame[22:21] <= S[1:0];
   L2frame[23]    <= P;
  end
 
 // Hamming code generation
 xor(H[0],D[0],D[1],D[3],D[4],D[6],D[8],D[10],D[11],D[13],D[15],S[1]);
 xor(H[1],D[0],D[2],D[3],D[5],D[6],D[9],D[10],D[12],D[13],S[0],S[1]);
 xor(H[2],D[1],D[2],D[3],D[7],D[8],D[9],D[10],D[14],D[15],S[0],S[1]);
 xor(H[3],D[4],D[5],D[6],D[7],D[8],D[9],D[10]);
 xor(H[4],D[11],D[12],D[13],D[14],D[15],S[0],S[1]); 
 // Parity code generation
 xor(P,H[0],H[1],H[2],H[3],H[4],D[0],D[1],D[2],D[3],D[4],D[5],D[6],
     D[7],D[8],D[9],D[10],D[11],D[12],D[13],D[14],D[15],S[0],S[1]);

endmodule 